from sqlalchemy import Column, Integer, String
from sqlalchemy.orm import Mapped

from subarashii.db import Base


class SubarashiiTable(Base):
    __tablename__ = "subarashii_table"

    id: Mapped[int] = Column(Integer, primary_key=True, index=True)

    subarashii_field: Mapped[str] = Column(String)



from sqlalchemy.orm import Session

from subarashii.model.subarashii_table import SubarashiiTable
from subarashii.repository import CRUDRepository


def test_crud_repository_save_all(session: Session):
    repository = CRUDRepository(session, SubarashiiTable)

    items = [
        SubarashiiTable(subarashii_field="subarashii value"),
        SubarashiiTable(subarashii_field="another subarashii value"),
        SubarashiiTable(subarashii_field="another another subarashii value"),
    ]

    repository.save_all(items)

    page = repository.get_all()

    assert page.total == len(items)

    for i in range(0, page.total):
        assert items[i].subarashii_field == page.elements[i].subarashii_field



from os import getenv

from dotenv import load_dotenv
from sqlalchemy import create_engine
from sqlalchemy.orm import declarative_base, sessionmaker

load_dotenv()

DB_USER = getenv("DB_USER")

DB_PASSWORD = getenv("DB_PASSWORD")

DB_HOST = getenv("DB_HOST")

DB_PORT = getenv("DB_PORT")

DB_NAME = getenv("DB_NAME")

DATABASE_URL = f"postgresql+psycopg2://{DB_USER}:{DB_PASSWORD}@{DB_HOST}:{DB_PORT}/{DB_NAME}"

engine = create_engine(DATABASE_URL, echo=True, future=True)

SessionLocal = sessionmaker(bind=engine, autoflush=False, autocommit=False, future=True)

Base = declarative_base()



from sqlalchemy.orm import Session

from subarashii.model.subarashii_table import SubarashiiTable
from subarashii.repository import CRUDRepository


def test_crud_repository_exists_by_id(session: Session):
    repository = CRUDRepository(session, SubarashiiTable)

    result = repository.save(SubarashiiTable(subarashii_field="subarashii value"))

    assert repository.exists_by_id(result.id) is not None

    assert repository.exists_by_id(result.id + 1) is not None



from sqlalchemy.orm import Session

from subarashii.model.subarashii_table import SubarashiiTable
from subarashii.repository import CRUDRepository


def test_crud_repository_get_by_id_returning_none(session: Session):
    repository = CRUDRepository(session, SubarashiiTable)

    result = repository.get_by_id(1)

    assert result is None


def test_crud_repository_get_by_id(session: Session):
    repository = CRUDRepository(session, SubarashiiTable)

    result = repository.save(SubarashiiTable(subarashii_field="subarashii value"))

    result = repository.get_by_id(result.id)

    assert result is not None



from pytest import raises
from sqlalchemy.orm import Session

from subarashii.exception.entity_not_found_exception import EntityNotFoundException
from subarashii.model.subarashii_table import SubarashiiTable
from subarashii.repository import CRUDRepository


def test_crud_repository_delete_throws_entity_not_found_exception(session: Session):
    repository = CRUDRepository(session, SubarashiiTable)

    with raises(EntityNotFoundException):
        repository.delete(1)


def test_crud_repository_delete(session: Session):
    repository = CRUDRepository(session, SubarashiiTable)

    expected_length = repository.get_all().total

    expected = repository.save(SubarashiiTable(subarashii_field="subarashii value"))

    repository.delete(expected.id)

    assert repository.get_all().total == expected_length



from typing import TypeVar, Type, Iterable, Optional, List

from sqlalchemy import exists
from sqlalchemy.orm import Session

from subarashii import Pageable, Page
from subarashii.exception.attribute_not_found_exception import AttributeNotFoundException
from subarashii.exception.entity_not_found_exception import EntityNotFoundException
from subarashii.repository.repository import Repository

ID = TypeVar("ID")

ENTITY = TypeVar("ENTITY")


class CRUDRepository(Repository[ENTITY, ID]):
    def __init__(self, session: Session, model: Type[ENTITY]) -> None:
        self.session = session

        self.model = model

    def save(self, entity: ENTITY) -> ENTITY:
        self.session.add(entity)

        self.session.commit()

        self.session.refresh(entity)

        return entity

    def save_all(self, iterable: Iterable[ENTITY]) -> Iterable[ENTITY]:
        self.session.add_all(iterable)

        self.session.commit()

        return iterable

    def get_all(self, pageable: Optional[Pageable] = None) -> Page[ENTITY]:
        query = self.session.query(self.model)

        total = query.count()

        if pageable:
            query = query.offset(pageable.page * pageable.size).limit(pageable.size)

            elements = query.all()

            return Page(elements, total, pageable.page, pageable.size)
        else:
            elements = query.all()

            return Page(elements, total, 0, total)

    def get_by_id(self, id: ID) -> Optional[ENTITY]:
        return self.session.get(self.model, id)

    def get_by(self, **kwargs) -> List[ENTITY]:
        if not kwargs:
            raise ValueError("at least one field must be provided for filtering")

        query = self.session.query(self.model)

        for key in kwargs:
            if not hasattr(self.model, key):
                raise AttributeNotFoundException()

        query = query.filter_by(**kwargs)

        return query.all()

    def update(self, id: ID, entity: ENTITY) -> ENTITY:
        existing = self.get_by_id(id)

        if not existing:
            raise EntityNotFoundException()

        for attr, value in vars(entity).items():
            if attr.startswith("_"):
                continue

            setattr(existing, attr, value)

        self.session.commit()

        self.session.refresh(existing)

        return existing

    def delete(self, id: ID) -> None:
        entity = self.get_by_id(id)

        if not entity:
            raise EntityNotFoundException()

        self.session.delete(entity)

        self.session.commit()

    def delete_all(self) -> None:
        self.session.query(self.model).delete()

        self.session.commit()

    def count(self) -> int:
        return self.session.query(self.model).count()

    def exists_by_id(self, id: ID) -> bool:
        return self.session.query(exists().where(self.model.id == id)).scalar()



from sqlalchemy.orm import Session

from subarashii.model.subarashii_table import SubarashiiTable
from subarashii.repository import CRUDRepository


def test_crud_repository_count(session: Session):
    repository = CRUDRepository(session, SubarashiiTable)

    count = repository.count()

    assert count == 0

    repository.save(SubarashiiTable(subarashii_field="subarashii value"))

    count = repository.count()

    assert count == 1

    repository.save(SubarashiiTable(subarashii_field="another subarashii value"))

    count = repository.count()

    assert count == 2



from sqlalchemy.orm import Session

from subarashii.model.subarashii_table import SubarashiiTable
from subarashii.repository import CRUDRepository


def test_crud_repository_delete_all(session: Session):
    repository = CRUDRepository(session, SubarashiiTable)

    repository.save(SubarashiiTable(subarashii_field="subarashii value"))

    repository.save(SubarashiiTable(subarashii_field="another subarashii value"))

    repository.save(SubarashiiTable(subarashii_field="another another subarashii value"))

    page = repository.get_all()

    start = page.total

    repository.delete_all()

    page = repository.get_all()

    end = page.total

    assert page.total == 0

    assert start != end



from pytest import raises
from sqlalchemy.orm import Session

from subarashii.exception.entity_not_found_exception import EntityNotFoundException
from subarashii.model.subarashii_table import SubarashiiTable
from subarashii.repository import CRUDRepository


def test_crud_repository_update_throws_entity_not_found_exception(session: Session):
    repository = CRUDRepository(session, SubarashiiTable)

    with raises(EntityNotFoundException):
        repository.update(1, SubarashiiTable(subarashii_field="subarashii value"))


def test_crud_repository_update(session: Session):
    repository = CRUDRepository(session, SubarashiiTable)

    expected = repository.save(SubarashiiTable(subarashii_field="subarashii value"))

    repository.update(expected.id, SubarashiiTable(subarashii_field="another subarashii value"))

    result = repository.get_by_id(expected.id)

    assert result is not None

    assert result.id == expected.id

    assert result.subarashii_field != "subarashii value"



from sqlalchemy.orm import Session

from subarashii.model.subarashii_table import SubarashiiTable
from subarashii.repository import CRUDRepository


def test_crud_repository_get_all_returns_empty_list(session: Session):
    repository = CRUDRepository(session, SubarashiiTable)

    page = repository.get_all()

    assert page.elements == []

    assert page.total == 0


def test_crud_repository_get_all_returns_not_empty_list(session: Session):
    repository = CRUDRepository(session, SubarashiiTable)

    repository.save(SubarashiiTable(subarashii_field="subarashii value"))

    page = repository.get_all()

    assert page.elements != []

    assert page.total != 0



from abc import ABC, abstractmethod
from typing import TypeVar, Generic, Iterable, Optional, List

from subarashii import Page, Pageable

ID = TypeVar("ID")

ENTITY = TypeVar("ENTITY")


class Repository(Generic[ENTITY, ID], ABC):
    @abstractmethod
    def save(self, entity: ENTITY) -> ENTITY:
        pass

    @abstractmethod
    def save_all(self, iterable: Iterable[ENTITY]) -> Iterable[ENTITY]:
        pass

    @abstractmethod
    def get_all(self, pageable: Optional[Pageable] = None) -> Page[ENTITY]:
        pass

    @abstractmethod
    def get_by_id(self, id: ID) -> Optional[ENTITY]:
        pass

    @abstractmethod
    def get_by(self, **kwargs) -> List[ENTITY]:
        pass

    @abstractmethod
    def update(self, id: ID, entity: ENTITY) -> ENTITY:
        pass

    @abstractmethod
    def delete(self, id: ID) -> None:
        pass

    @abstractmethod
    def delete_all(self) -> None:
        pass

    @abstractmethod
    def count(self) -> int:
        pass

    @abstractmethod
    def exists_by_id(self, id: ID) -> bool:
        pass



from pytest import raises
from sqlalchemy.orm import Session

from subarashii.exception.attribute_not_found_exception import AttributeNotFoundException
from subarashii.model.subarashii_table import SubarashiiTable
from subarashii.repository import CRUDRepository


def test_crud_repository_get_by_throws_value_error(session: Session):
    repository = CRUDRepository(session, SubarashiiTable)

    with raises(ValueError) as e:
        repository.get_by()

    assert e.value.args[0] == "at least one field must be provided for filtering"


def test_crud_repository_get_by_throws_attribute_not_found(session: Session):
    repository = CRUDRepository(session, SubarashiiTable)

    with raises(AttributeNotFoundException) as e:
        repository.get_by(not_subarashii_field="not subarashii value")

    assert e.value.args[0] == "attribute not found"


def test_crud_repository_get_by(session: Session):
    repository = CRUDRepository(session, SubarashiiTable)

    repository.save(SubarashiiTable(subarashii_field="subarashii value"))

    repository.save(SubarashiiTable(subarashii_field="another subarashii value"))

    repository.save(SubarashiiTable(subarashii_field="another another subarashii value"))

    repository.save(SubarashiiTable(subarashii_field="another another another subarashii value"))

    result = repository.get_by(subarashii_field="subarashii value")

    assert len(result) == 1

    assert result[0].subarashii_field == "subarashii value"



from sqlalchemy.orm import Session

from subarashii.model.subarashii_table import SubarashiiTable
from subarashii.repository import CRUDRepository


def test_crud_repository_save(session: Session):
    repository = CRUDRepository(session, SubarashiiTable)

    result = repository.get_all()

    assert result.elements == []

    assert result.total == 0

    result = repository.save(SubarashiiTable(subarashii_field="subarashii value"))

    assert result is not None

    result = repository.get_by_id(result.id)

    assert result is not None

    page = repository.get_all()

    assert page.elements

    assert page.total == 1



import pytest
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

from subarashii.db import Base

TEST_DATABASE_URL = "sqlite:///:memory:"


@pytest.fixture(scope="function")
def session():
    engine = create_engine(TEST_DATABASE_URL, echo=False, future=True)

    Base.metadata.create_all(bind=engine)

    TestingSessionLocal = sessionmaker(bind=engine, autoflush=False, autocommit=False, future=True)

    session = TestingSessionLocal()

    try:
        yield session
    finally:
        session.close()



from typing import Generic, List, TypeVar

from subarashii.db import SessionLocal
from subarashii.exception.attribute_not_found_exception import AttributeNotFoundException
from subarashii.exception.entity_not_found_exception import EntityNotFoundException

ID = TypeVar("ID")

ENTITY = TypeVar("ENTITY")


class Pageable:
    def __init__(self, page: int = 0, size: int = 10):
        self.page = max(page, 0)

        self.size = max(size, 1)

    def __str__(self):
        return f"Pageable(page={self.page}, size={self.size})"


class Page(Generic[ENTITY]):
    def __init__(self, elements: List[ENTITY], total: int, page: int, size: int):
        self.elements = elements

        self.total = total

        self.page = page

        self.size = size

    def __str__(self):
        return f"Page(page={self.page}, size={self.size}, total={self.total}, elements={self.elements})"
